/** \file main.c
 *
 * \brief Timer interrupt.
 *
 * This example program switches LEDs on and off using a timer interrupt.  
 *
 * The relevant registers for the timer are:
 * - CTRLA: Timer Control Register A
 *   - CLKSEL: Clock Select (enable and prescaler)
 * - INTCTRLA: Timer Interrupt Enable Register A
 *   - OVFINTLVL: Timer Overflow Interrupt Level
 * - PER: Timer Period Register
 *
 * The following registers are used to enable the timer interrupt:
 * - CPU_SREG: CPU Status Register
 *   - I: Global Interrupt Enable
 * - PMIC.CTRL: Programmable Multi-Level Interrupt Controller Register
 *   - LOLVLEN: Low-Level Interrupt Enable
 *
 * The value for PER is calculated by PER = F_CPU/(n*TPS) - 1 where
 * - n: Prescale value
 * - TPS: Ticks per seconds
 * - F_CPU: System frequency
 *
 * Please see modules 4.3.2 and 4.3.4 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-06-30
 *
 * \par History
 *      Created: 2013-07-16 \n
 *      Modified: 2018-06-30
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

// We want LED0 to blink with a frequency of 1 Hz. Blinking means that half of
// the time the LED is on and half of the time the LED is off. Therefore we need
// 2 interrupts per second to toggle the LED. We use a prescaler of 64. So we need
// a PER of 15624 (n=64, TPS=2, F_CPU=2000000). We use timer 1 of port F and will
// generate low level interrupts. As the job is done by the interrupt service
// routine the super loop has nothing to do.
int main(void)
{
    // Initialize the LED.
    LED_PORT.DIR = LED0_PIN_bm;
    LED_PORT.OUT = LED0_PIN_bm;

	// Initialize the timer.
	TCF1.PER = 15624;
    TCF1.INTCTRLA = TC_OVFINTLVL_LO_gc;
	TCF1.CTRLA = TC_CLKSEL_DIV64_gc;
	
	// Enable low level interrupts.
	PMIC.CTRL |= PMIC_LOLVLEN_bm;
    CPU_SREG |= CPU_I_bm;

    while(1)
    {
        // Nothing to do.
    }
}

/***** Timer/counter overflow interrupt *****/
// This interrupt service routine gets called two times per second.
ISR(TCF1_OVF_vect)
{
	// Toggle the LED.
	LED_PORT.OUT ^= LED0_PIN_bm;
}
