/** \file main.c
 *
 * \brief USART interrupt.
 *
 * This example program receives characters from the console and sends them back.
 *
 * The relevant registers are:
 * - CTRLA: USART Control Register A
 *   - RXCINTLVL: Receive Complete Interrupt Level
 *
 * The following registers are used to enable the timer interrupt:
 * - CPU_SREG: CPU Status Register
 *   - I: Global Interrupt Enable
 * - PMIC.CTRL: Programmable Multi-Level Interrupt Controller Register
 *   - LOLVLEN: Low-Level Interrupt Enable
 *
 * \note
 * This program implements example '04. USART' of the peripherals course with the
 * aid of a receive complete interrupt. Only the interrupt part is explained here.
 * Please see the peripherals course for more information on the USART part.
 *
 * \warning
 * This program demonstrates the use of the receive complete interrupt.
 * It is not correct as there is a race condition. Race conditions are
 * discussed in the next module and not taken into account here. Please
 * see the advanced example for a correct version of the program.
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-06-30
 *
 * \par History
 *      Created: 2013-07-16 \n
 *      Modified: 2018-06-30
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

#define BUFFER_FULL 1
#define BUFFER_EMPTY 0

// Variables which are changed by interrupts must be declared as volatile.
volatile uint8_t buffer;
volatile uint8_t status = BUFFER_EMPTY;

// We want to send back the characters which we receive via the USART interface.
// We use the receive complete interrupt to handle the reception and store the
// character received in a buffer. If the buffer is full we send it back in
// the super loop.
int main(void)
{
    // Initialize the USART pins.
	USB_USART_PORT.OUT |= USB_USART_TX_PIN_bm;
	USB_USART_PORT.DIR |= USB_USART_TX_PIN_bm;
	USB_USART_PORT.DIR &= ~USB_USART_RX_PIN_bm;

    // Initialize USART and enable RXC interrupt.
	USB_USART_MODULE.CTRLA = USART_RXCINTLVL_LO_gc;
	USB_USART_MODULE.CTRLC = (USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc);
	USB_USART_MODULE.BAUDCTRLA = USB_USART_BSEL;
	USB_USART_MODULE.BAUDCTRLB = (USB_USART_BSCALE << USART_BSCALE_gp);
	USB_USART_MODULE.CTRLB = (USART_RXEN_bm | USART_TXEN_bm);

	// Enable low-level interrupts.
	PMIC.CTRL |= PMIC_LOLVLEN_bm;
	CPU_SREG |= CPU_I_bm;

	while(1)
	{
		// Echo data if buffer is full.
		if (status == BUFFER_FULL) {
			// Wait until data register is empty.
			while(!(USB_USART_MODULE.STATUS & USART_DREIF_bm));
			// Send character stored in buffer.
			USB_USART_MODULE.DATA = buffer;
			// The buffer is now empty again.
            status = BUFFER_EMPTY;
		}
	}
}

/*****  USART receive complete interrupt *****/
// This interrupt service routine gets called when new data has been received.
ISR(USB_USART_RXC_vect)
{
	// Store character in buffer.
	buffer = USB_USART_MODULE.DATA;
	// Say that buffer is full.
	status = BUFFER_FULL;
}
