/** \file main.c
 *
 * \brief USART interrupt - advanced version.
 *
 * This version of '02 USART' is advanced training material for experienced students.
 *
 * Race conditions occur if data access is interrupted. In order to prevent race
 * conditions data access must be atomic. This means that it must not be interrupted.
 * One way to make data access atomic is to disable the interrupts. Disabling interrupts
 * is a bad idea. They should be disabled for a very short, only. This example shows
 * how this can be done.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-06-30
 *
 * \par History
 *      Created: 2018-06-30
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

#define BUFFER_FULL 1
#define BUFFER_EMPTY 0

// Compilers are allowed to optimize code. They suppose that variables
// can never change unexpectedly. This is not true if variables are
// modified by interrupts. Therefore such variables have to be
// declared volatile.
volatile uint8_t buffer;
volatile uint8_t status = BUFFER_EMPTY;

// We want to send back the characters which we receive via the USART interface.
// We use the receive complete interrupt to handle the reception and store the
// character received in a buffer. If the buffer is full we send it back in
// the super loop.
int main(void)
{
    // Initialize the USART pins.
	USB_USART_PORT.OUT |= USB_USART_TX_PIN_bm;
	USB_USART_PORT.DIR |= USB_USART_TX_PIN_bm;
	USB_USART_PORT.DIR &= ~USB_USART_RX_PIN_bm;

    // Initialize the USART and enable RXC interrupt.
	USB_USART_MODULE.CTRLA = USART_RXCINTLVL_LO_gc;
	USB_USART_MODULE.CTRLC = (USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc);
	USB_USART_MODULE.BAUDCTRLA = USB_USART_BSEL;
	USB_USART_MODULE.BAUDCTRLB = (USB_USART_BSCALE << USART_BSCALE_gp);
	USB_USART_MODULE.CTRLB = (USART_RXEN_bm | USART_TXEN_bm);

	// Enable low-level interrupts.
	PMIC.CTRL |= PMIC_LOLVLEN_bm;
	CPU_SREG |= CPU_I_bm;

	while(1)
	{
		// Echo data if buffer is full.
		if (status == BUFFER_FULL) {
			/***** Atomic section starts here. *****/
			cli(); // Disable interrupts.
			// The period where the interrupts are disabled must be as short as possible.
			// We can not wait until the data register of the USART is empty (DRE).
			// Therefore we store the content of the buffer in a temporary variable.
			uint8_t temp = buffer;
			// We saved the content. The buffer is free again and can get new data.
			status = BUFFER_EMPTY;
			sei(); // Enable interrupts
			/***** Atomic section ends here. *****/
			// Interrupts are enabled again. Now we have time to wait.
			while(!(USB_USART_MODULE.STATUS & USART_DREIF_bm));
			// Send character stored in temporary buffer.
			USB_USART_MODULE.DATA = temp;
		}
	}
}

/*****  USART receive complete interrupt *****/
// This interrupt service routine gets called when new data has been received.
ISR(USB_USART_RXC_vect)
{
	// Store character in buffer.
	buffer = USB_USART_MODULE.DATA;
	// Say that buffer is full.
	status = BUFFER_FULL;
}
