/** \file main.c
 *
 * \brief Port interrupt.
 *
 * This example program checks if push-buttons are pressed.
 *
 * The relevant registers are:
 * - INTCTRL: Interrupt Control Register
 *   - INT1LVL: Interrupt 0 Level
 *   - INT0LVL: Interrupt 1 Level
 * - INT0MASK: Interrupt 0 Mask Register
 * - INT1MASK: Interrupt 1 Mask Register
 *
 * \note
 * Ports can throw interrupts. There are two tow interrupts per port:
 * INT0 and INT1. A mask register maps the pins with the two possible
 * interrupts.
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-06-30
 *
 * \par History
 *      Created: 2016-10-13 \n
 *      Modified: 2018-06-30
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

// We want to switch LED0 on if button 0 is pressed and
// we want to switch it off if button 1 is pressed.
int main(void)
{
	// Initialize the LED.
    LED_PORT.DIR = LED0_PIN_bm;
    LED_PORT.OUT = LED0_PIN_bm;

	/***** Initialize push-buttons and enable interrupts. *****/
	// Enable port interrupt 0 and 1.
	BUTTON_LOW_PORT.INTCTRL = PORT_INT0LVL_LO_gc | PORT_INT1LVL_LO_gc;
	// Pin 0 triggers interrupt 0
	BUTTON_LOW_PORT.INT0MASK = BUTTON0_PIN_bm;
	// Pin 1 triggers interrupt 1
	BUTTON_LOW_PORT.INT1MASK = BUTTON1_PIN_bm;
	// Trigger interrupt on falling edge
	BUTTON_LOW_PORT.PIN0CTRL = PORT_OPC_PULLUP_gc | PORT_ISC_FALLING_gc;
	BUTTON_LOW_PORT.PIN1CTRL = PORT_OPC_PULLUP_gc | PORT_ISC_FALLING_gc;

	// Enable low-level interrupts. 
	PMIC.CTRL |= PMIC_LOLVLEN_bm;
	CPU_SREG |= CPU_I_bm;

	while(1)
	{
        // Nothing to do
	}
}

/***** Port interrupt 0 *****/
// This interrupt service routine gets called whenever port interrupt 0 is thrown.
ISR(PORTD_INT0_vect)
{
	// Switch the LED on.
	LED_PORT.OUT &= ~LED0_PIN_bm;
}

/***** Port interrupt 1 *****/
// This interrupt service routine gets called whenever port interrupt 1 is thrown.
ISR(PORTD_INT1_vect)
{
	// Switch the LED off
	LED_PORT.OUT |= LED0_PIN_bm;
}
