/** \file main.c
 *
 * \brief Direct Memory Access
 *
 * This example program transfers a long message via USART with the aid of DMA.
 *
 * This example is advanced training material for experienced students.
 *
 * In order to transfer a long message via the USART a loop is needed. The loop 
 * must be handled by the CPU and it has to transfer the message one character at
 * a time. A DMA controller can relieve the CPU. It loops through the message and
 * sends the individual characters to the USART. Programming the DMA controller,
 * however, is not that simple.
 *
 * The relevant registers are:
 * - CTRL: Control Register
 *   - ENABLE: Enable DMA controller
 * - CHn.CTRLA: Channel n Control Register A
 *   - ENABLE: Enable Channel n
 *   - SINGLE: Single-Shot Data Transfer
 *   - BURSTLEN: Burst Mode
 * - CHn.ADDRCTRL: Channel n Address Control Register
 *   - SRCRELOAD: Source Address Reload
 *   - SRCDIR: Source Address Mode
 *   - DESTRELOAD: Destination Address Reload
 *   - DESTDIR: Destination Address Mode
 * - CHn.TRIGSRC: Channel n Trigger Source Register
 * - CHn.TRFCNT: Channel n Block Transfer Count Register
 * - CHn.SRCADDR0: Channel n Source Address Register 0 (byte 0 of source address)
 * - CHn.SRCADDR1: Channel n Source Address Register 1 (byte 1 of source address)
 * - CHn.SRCADDR2: Channel n Source Address Register 2 (byte 2 of source address)
 * - CHn.DESTADDR0: Channel n Destination Address Register 0 (byte 0 of destination address)
 * - CHn.DESTADDR1: Channel n Destination Address Register 1 (byte 1 of destination address)
 * - CHn.DESTADDR2: Channel n Destination Address Register 2 (byte 2 of destination address)
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-07-01
 *
 * \par History
 *      Created: 2016-10-14 \n
 *      Modified: 2018-07-01
 */

#include <string.h>
#include <avr/io.h>
#include <util/delay.h>

#include "board.h"

#define LOWBYTE(w) (((uint16_t)(w) >> 0) & 0xFF)
#define HIGHBYTE(w) (((uint16_t)(w) >> 8) & 0xFF)

int main(void)
{
	char message[] = "This is a somewhat longer message.\r\n";

	// Initialize the USART.
	USB_USART_PORT.OUT = USB_USART_TX_PIN_bm;
	USB_USART_PORT.DIR = USB_USART_TX_PIN_bm;
	USB_USART_MODULE.CTRLC = (USART_CMODE_ASYNCHRONOUS_gc | USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc);
	USB_USART_MODULE.BAUDCTRLA = 11;
	USB_USART_MODULE.BAUDCTRLB = (-7<<USART_BSCALE_gp);
	USB_USART_MODULE.CTRLB = USART_TXEN_bm;

	/***** Initialize DMA controller. *****/
	// Enable the DMA controller.
    DMA.CTRL |= DMA_ENABLE_bm;
    // Set the source address.
	DMA.CH0.SRCADDR0 = LOWBYTE(message);
	DMA.CH0.SRCADDR1 = HIGHBYTE(message);
	DMA.CH0.SRCADDR2 = 0;
    // Set the destination address.
	DMA.CH0.DESTADDR0 = LOWBYTE(&USB_USART_MODULE.DATA);
	DMA.CH0.DESTADDR1 = HIGHBYTE(&USB_USART_MODULE.DATA);
	DMA.CH0.DESTADDR2 = 0;

    /***** Initialize the DMA channel. *****/
	// Increment source address and reload source if block transfer has finished.
	// Do not modify destination address and do not reload destination if transfer has finished.
	DMA.CH0.ADDRCTRL = DMA_CH_SRCRELOAD_BLOCK_gc | DMA_CH_SRCDIR_INC_gc | DMA_CH_DESTRELOAD_NONE_gc | DMA_CH_DESTDIR_FIXED_gc;
	// Burst length is 1 byte and stop after each byte.
	DMA.CH0.CTRLA = DMA_CH_BURSTLEN_1BYTE_gc | DMA_CH_SINGLE_bm;
	// Set block length.
	DMA.CH0.TRFCNT =strlen(message);
	// Set the trigger source.
	DMA.CH0.TRIGSRC = DMA_CH_TRIGSRC_USARTC0_DRE_gc;

	while(1)
	{
		// Start the transmission.
        DMA.CH0.CTRLA |= DMA_CH_ENABLE_bm;
		// Wait a bit before the message is sent again.
		_delay_ms(1000);
	}
}
